package functionaljavatest;

import fj.F;
import fj.F2;
import fj.P;
import fj.P2;
import fj.P2;
import fj.data.Array;
import functionaljavatest.Pnt;
import functionaljavatest.Vec;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

/**
 *
 * @author ubik
 */
public class Game {

    private GameCanvas canvas;
    private Array<Pnt> position;
    private Array<Vec> velocity;
    private Array<F2<Pnt, Vec, Vec>> equations;

    public Game(GameCanvas canvas) {
        this.canvas = canvas;

        this.position = Array.empty();
        this.velocity = Array.empty();
        this.equations = Array.empty();

        Timer timer = new Timer();
        timer.scheduleAtFixedRate(new TimerTask() {
            @Override
            public void run() {
                tick();
            }
        }, 100, 25);
    }

    public void addEquation(F2<Pnt, Vec, Vec> eq) {
        equations = equations.append(Array.single(eq));
    }

    public void addSprite(Pnt pos) {
        synchronized (this) {
            Vec vel = new Vec((Math.random() * 6 - 3), (Math.random() * 6 - 3));
            position = position.append(Array.single(pos));
            velocity = velocity.append(Array.single(vel));
        }
    }

    public void tick() {
        // Update positions
        position = position.zip(velocity).map(P2.tuple(Pnt.padd));

        F2<Array<P2<Pnt, Vec>>, F2<Pnt, Vec, Vec>, Array<P2<Pnt, Vec>>> fold 
                = new F2<Array<P2<Pnt, Vec>>, F2<Pnt, Vec, Vec>, Array<P2<Pnt, Vec>>>() {
                    
            @Override
            public Array<P2<Pnt, Vec>> f(Array<P2<Pnt, Vec>> values, F2<Pnt, Vec, Vec> equation) {
                Array<Vec> updatedVel = values.map( P2.tuple(equation));
                
                return values.zip(updatedVel).map(new F< P2< P2<Pnt,Vec> ,Vec>,P2<Pnt,Vec> >() {
                    @Override
                    public P2<Pnt, Vec> f(P2<P2<Pnt, Vec>, Vec> zipped) {
                        return P.p( zipped._1()._1(), zipped._2() );
                    }
                } );
            }
        };
        
        F<P2<Pnt, Vec>, Vec> reduce = P2.<Pnt,Vec>__2();
        velocity = equations.foldLeft(fold, position.zip(velocity)).map(reduce);

        
        fireChangeEvent();
    }

    private <A, B, C> F<P2<A, B>, C> wrap(final F2<A, B, C> func) {
        return new F<P2<A, B>, C>() {

            @Override
            public C f(P2<A, B> a) {
                return func.f(a._1(), a._2());
            }
        };
    }

    public ArrayList<Pnt> getPositions() {
        return new ArrayList<Pnt>(position.toCollection());
    }

    private void fireChangeEvent() {
        canvas.redraw();
    }
}
